/* Copyright (C) 2015 by Marcel Mueller <marcel@heilpraktiker-praxis-chemnitz.de>
*   All rights reserved.
*
*    This program is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License \n
*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*/ /*! \file *******************************************************************
*
* $File:	    hih-8131.c $
* $Created: 	26.02.2015 $
* $Revised: 	26.02.2015 $
* $Version: 	1.0 $
* $Author: 	    Marcel Mueller $
* $Target:   	any AVR device $
* $Compiler:	AVR-GCC 4.8.2 / avr-libc 1.8.0 $
* $License:	    GPLv3 or later $
*
* @par Description:
*		Source File for the Honeywell HumidIconTM Digital
*		Humidity/Temperature Sensors.
*
*******************************************************************************/



/* I N C L U D E   F I L E S
*******************************************************************************/
#include "i2cmaster.h"
#include "hih-8131.h"


/* M A C R O   D E F I N I T I O N S
*******************************************************************************/
#define I2C_ADDR 	0x4E

#define ACK		    1
#define NACK		0

#define TRUE		1
#define FALSE		0

#define I2C_READ    1
#define I2C_WRITE   0

/* T Y P E   D E F I N I T I O N S
*******************************************************************************/



/* G L O B A L   V A R I A B L E S
*******************************************************************************/
uint8_t i2c_addr = I2C_ADDR;


/* P R O T O T Y P E S
*******************************************************************************/



/* L O C A L   F U N C T I O N S
*******************************************************************************/



/* P U B L I C   F U N C T I O N S
*******************************************************************************/


/*
*
*	@brief 	Humidity and Temperature Measurement Request
*
*	@fn 	void hih_mr(void)
*
*******************************************************************************/
void hih_mr(void){
i2c_start(i2c_addr + I2C_WRITE);
i2c_write(0x00);
i2c_stop();
}


/*
*
*	@brief 	Humidity and Temperature Data Fetch
*
*	@fn 	uint8_t hih_df(uint16_t *hum, uint16_t *tmp)
*	@param  hum a Pointer to Humidity Data 14 Bit
*	@param  tmp a Pointer to Temperature Data 14 Bit
*	@retval 0   Normal Operation
*	@retval 1   Stale Data
*	@retval 2   Device in Command Mode
*	@retval 3   Not Used
*
*******************************************************************************/
uint8_t hih_df(uint16_t *hum, uint16_t *tmp){

uint8_t data_byte[4];
i2c_start(i2c_addr + I2C_READ);
data_byte[0] = i2c_read(ACK);	//humH + status
data_byte[1] = i2c_read(ACK);	//humL
data_byte[2] = i2c_read(ACK);	//tmpH
data_byte[3] = i2c_read(NACK);	//tmpL
i2c_stop();

*hum   = (((uint16_t)(0x3F & data_byte[0]))<<8) | data_byte[1];
*tmp   = ((((uint16_t)data_byte[2])<<8) | data_byte[3])>>2;

return (data_byte[0]>>6);	//status
}


/*
*
*	@brief	Scan the i2C Address, only for a Single Device on Bus
*
*	@fn 	uint8_t hih_scanaddr(void)
*	@return i2c Address
*
*******************************************************************************/
uint8_t hih_scanaddr(void){
uint8_t addr, count, data_byte[4];

data_byte[0] = 0xFF;
count = 0;
addr  = 0;
_delay_ms(10);
while((data_byte[0] == 0xFF) && (count <= 0x7F)){
i2c_start(count + I2C_READ);
data_byte[0] = i2c_read(ACK);
data_byte[1] = i2c_read(ACK);
data_byte[2] = i2c_read(ACK);
data_byte[3] = i2c_read(NACK);
i2c_stop();
addr = count;
count++;
}
return addr;
}


/*
*
*	@brief 	Set the i2C Address
*
*	@fn 	void hih_setaddr(uint8_t addr)
*	@param  i2c Address
*
*******************************************************************************/
void hih_setaddr(uint8_t addr){
i2c_addr = addr;
}


/*
*
*	@brief 	Make a complete Measurement Cycle and convert RH in % and T in C°
*
*	@fn 	uint8_t hih_getrht(float *rh, float *t)
*	@param  rh a Pointer to Relativ Humidity Data in %
*	@param  t a Pointer to Temperature Data in C°
*	@retval <0x40   Normal Operation
*	@retval  0x7F   Stale Data
*	@retval  0xBF   Device in Command Mode
*	@retval  0xFF   Not Used
*
*******************************************************************************/
uint8_t hih_getrht(float *rh, float *t){

uint16_t _rh, _t; uint8_t status, poll_count=0;

hih_mr();
do
	{
	_delay_ms(2);//pool limit 126ms
	status = hih_df(&_rh, &_t);
	}
	while ((status) && (poll_count++ <= 0x3F));

	*t   = (((float)_t) * 1.007e-2 - 40.0);
	*rh = (((float)_rh) * 6.10e-3);


return ((status<<6) | poll_count);	//[S][S][C][C][C][C][C][C]
}

