/* Copyright (C) 2015 by Marcel Mueller <marcel@heilpraktiker-praxis-chemnitz.de>
*   All rights reserved.
*
*    This program is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License \n
*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*/ /*! \file *******************************************************************
*
* $File:	hih-8131.h $
* $Created: 	26.02.2015 $
* $Revised: 	26.02.2015 $
* $Version: 	1.0 $
* $Author: 	Marcel Mueller $
* $Target:   	any AVR device $
* $Compiler:	AVR-GCC 4.8.2 / avr-libc 1.8.0 $
* $License:	GPLv3 or later $
*
* @par Description: 	
*		Header File for the Honeywell HumidIconTM Digital
*		Humidity/Temperature Sensors.
*
*
*******************************************************************************/
#ifndef __HIH_8131_H
#define __HIH_8131_H 1


/*! \defgroup hih-8131
*
*	Communication with the Honeywell HumidIconTM Digital Humidity/Temperature
*	Sensors. Use's the I2C Master library from Peter Fleury <pfleury@gmx.ch> 
*	http://jump.to/fleury
*
*	Example usage:
*
*	@code 
*	#include "hih-8131.h" 
*
*	
*	void main(void){
*	
*	hih_setaddr(hih_scanaddr());
*	
*	float hr, t;
*	uint8_t status = hih_getrht(&rh, &t);
*
*	if(status<0x40){ printf("RH = %f T = %f", rh, t);}
*	else {printf("ERROR %i,status);}
*
*	}
*	@endcode
*
*******************************************************************************/



/* I N C L U D E   F I L E S
*******************************************************************************/
#include <stdint.h>
#include <util/delay.h>


/* M A C R O   D E F I N I T I O N S
*******************************************************************************/



/* T Y P E   D E F I N I T I O N S
*******************************************************************************/



/* G L O B A L   V A R I A B L E S
*******************************************************************************/



/* P R O T O T Y P E S
*******************************************************************************/

/*! \ingroup hih-8131
*
*	@brief 	Humidity and Temperature Measurement Request
*
*	@fn 	void hih_mr(void)
*
*******************************************************************************/
void hih_mr(void);


										
/*! \ingroup hih-8131
*
*	@brief 	Humidity and Temperature Data Fetch
*
*	@fn 	uint8_t hih_df(uint16_t *hum, uint16_t *tmp)
*	@param  hum a Pointer to Humidity Data 14 Bit
*	@param  tmp a Pointer to Temperature Data 14 Bit
*	@retval 0   Normal Operation
*	@retval 1   Stale Data
*	@retval 2   Device in Command Mode
*	@retval 3   Not Used
*
*******************************************************************************/
uint8_t hih_df(uint16_t *hum, uint16_t *tmp);


/*! \ingroup hih-8131
*
*	@brief	Scan the i2C Address, only for a Single Device on Bus
*	
*	@fn 	uint8_t hih_scanaddr(void)
*	@return i2c Address
*
*******************************************************************************/
uint8_t hih_scanaddr(void);



/*! \ingroup hih-8131
*
*	@brief 	Set the i2C Address
*	
*	@fn 	void hih_setaddr(uint8_t addr)
*	@param  addr i2c Address
*
*******************************************************************************/
void hih_setaddr(uint8_t addr);



/*! \ingroup hih-8131
*
*	@brief 	Make a complete Measurement Cycle and convert RH in % and T in C°
*
*	@fn 	uint8_t hih_getrht(float *rh, float *t)
*	@param  rh a Pointer to Relativ Humidity Data in %
*	@param  t a Pointer to Temperature Data in C°
*	@retval <0x40   Normal Operation
*	@retval  0x7F   Stale Data
*	@retval  0xBF   Device in Command Mode
*	@retval  0xFF   Not Used
*
*******************************************************************************/
uint8_t hih_getrht(float *rh, float *t);


#endif
